#
# $Id: Darwin.pm,v 717225574cff 2015/11/12 08:46:57 gomor $
#
package Net::Routing::Darwin;
use strict;
use warnings;

our $VERSION = '0.44';

use base qw(Net::Routing::FreeBSD);

1;

__END__

=head1 NAME

Net::Routing::Darwin - manage route entries on Darwin

=head1 SYNOPSIS

=head1 AUTHOR

Patrice E<lt>GomoRE<gt> Auffret

=head1 COPYRIGHT AND LICENSE

Copyright (c) 2015, Patrice E<lt>GomoRE<gt> Auffret

You may distribute this module under the terms of the Artistic license.
See LICENSE.Artistic file in the source distribution archive.

=cut
